// Sample code: load two images, extract feature vectors and compare the similarity
// This sample is similar to face_recognition_image_buffer.cpp, and shows how to call setImage() with a jpeg file path.
// First two images of the same identity are compared. Then the similarity score of two different identities is computed.
// Build on Linux: g++ -o face_recognition --std=c++11 ./face_recognition.cpp -I ../ -L ../trueface_sdk -l tf -fopenmp
// Build on macOS: g++ -o face_recognition --std=c++11 ./face_recognition.cpp -I ../ -L ../trueface_sdk -l tf

#include "trueface_sdk/tf_sdk.h"
#include <iostream>
#include <vector>

int main() {
    Trueface::SDK tfSdk;
    // TODO: replace the string with your license code.
    bool valid = tfSdk.setLicense("${LICENSE_CODE}");

    if (valid == false) {
        std::cout<<"Error: the provided license is invalid."<<std::endl;
        return 1;
    }

    // Load the first image and extract the feature vector.
    Trueface::ErrorCode errorCode = tfSdk.setImage("../images/brad_pitt_1.jpg");
    if (errorCode != Trueface::ErrorCode::NO_ERROR) {
        std::cout<<"Error: could not load the image"<<std::endl;
        return 1;
    }

    Trueface::Faceprint faceprint1;
    bool foundFace;
    errorCode = tfSdk.getLargestFaceFeatureVector(faceprint1, foundFace);
    if (errorCode != Trueface::ErrorCode::NO_ERROR || !foundFace) {
        std::cout<<"Unable to detect face or generate faceprint"<<std::endl;
        return 1;
    }

    // Load the second image and extract the feature vector.
    errorCode = tfSdk.setImage("../images/brad_pitt_2.jpg");
    if (errorCode != Trueface::ErrorCode::NO_ERROR) {
        std::cout<<"Error: could not load the image 2"<<std::endl;
        return 1;
    }

    Trueface::Faceprint faceprint2;
    errorCode = tfSdk.getLargestFaceFeatureVector(faceprint2, foundFace);
    if (errorCode != Trueface::ErrorCode::NO_ERROR || !foundFace) {
        std::cout<<"Unable to detect face or generate faceprint"<<std::endl;
        return 1;
    }

    // Compute the similarity between the two face images.
    float similarityScore;
    float matchProbability;
    tfSdk.getSimilarity(faceprint1, faceprint2, matchProbability, similarityScore);

    std::cout<<"Similarity score of same identity images: "<<similarityScore<<std::endl;
    std::cout<<"Match probability of same identity images: "<<matchProbability<<std::endl;

    // Load the image of a different identity and extract the feature vector.
    errorCode = tfSdk.setImage("../images/tom_cruise_1.jpg");
    if (errorCode != Trueface::ErrorCode::NO_ERROR) {
        std::cout<<"Error: could not load the image 2"<<std::endl;
        return 1;
    }

    Trueface::Faceprint faceprint3;
    errorCode = tfSdk.getLargestFaceFeatureVector(faceprint3, foundFace);
    if (errorCode != Trueface::ErrorCode::NO_ERROR || !foundFace) {
        std::cout<<"Unable to detect face or generate faceprint"<<std::endl;
        return 1;
    }

    // Compute the similarity between the images of different identities.
    tfSdk.getSimilarity(faceprint1, faceprint3, matchProbability, similarityScore);

    std::cout<<"Similarity score of two different identities: "<<similarityScore<<std::endl;
    std::cout<<"Match probability of two different identities: "<<matchProbability<<std::endl;

    return 0;
}